
# UConv⥸塼ƥȤ롣

# С
#     uconv-0.4.12
# 
#     $ ruby extconf.rb --enable-fullwidth-reverse-solidus

require "uconv"

EUCJP_RT_CHARS = [
  # JIS X 0208
    "\x8e\xa1",
    "\x8e\xdf",
    "\xa1\xb1",  # OVERLINE
    "\xa1\xbd",  # EM DASH
    "\xa1\xc0",  # REVERSE SOLIDUS
    "\xa1\xc1",  # WAVE DASH
    "\xa1\xc2",  # DOUBLE VERTICAL LINE
    "\xa1\xd0",  # MINUS SIGN
    "\xa1\xef",  # YEN SIGN
    "\xa1\xf1",  # CENT SIGN
    "\xa1\xf2",  # POUND SIGN
    "\xa2\xcc",  # NOT SIGN

  # JIS X 0212 ()
  "\x8f\xa2\xc3", # BROKEN BAR

  # NECüʸ
    "\xad\xa1",
    "\xad\xdf",    # ʿ
    "\xad\xf9",

  # IBMĥʸ
  "\x8f\xf3\xf3",
  "\x8f\xf4\xfe"]

SJIS_RT_CHARS = [
  # JIS X 0208
    "\x81\x5C",    # EM DASH
    "\x81\x5f",    # REVERSE SOLIDUS
    "\x81\x60",    # WAVE DASH
    "\x81\x61",    # DOUBLE VERTICAL LINE
    "\x81\x7C",    # MINUS SIGN
    "\x81\x91",    # CENT SIGN
    "\x81\x92",    # POUND SIGN
    "\x81\xCA",    # NOT SIGN

  # NECüʸ
    "\x87\x40",  # 
    "\x87\x7e",  # ʿ

  # IBMĥ
    "\xfa\x40", # i
    "\xfc\x4b"]

EUCJP_TOUCS = [
  # NECüʸΤJIS X 0208Ƚʣ沽Ƥ
  ["\xad\xf0", 0x2252],
  ["\xad\xf1", 0x2261],
  ["\xad\xf2", 0x222b],
  ["\xad\xf5", 0x221a],
  ["\xad\xf6", 0x22a5],
  ["\xad\xf7", 0x2220],
  ["\xad\xfa", 0x2235],
  ["\xad\xfb", 0x2229],
  ["\xad\xfc", 0x222a],

  # JIS X 0212
  ["\x8f\xa2\xb7", 0xff5e], # TILDE -> U+FF5E FULLWIDTH TILDE

  # error
  ["\x8f\xff\xff", ??]
]

SJIS_TOUCS = [
  # NECüʸ
  ["\x87\x9b", 0x2229],
  ["\x87\x9c", 0x222a]
]

def pb(s)
  r = ""
  if s.is_a?(NilClass)
    r += "(nil) "
  elsif s.is_a?(Integer)
    r += sprintf("%02x ", s)
  else
    s.each_byte {|c|
      r += sprintf("%02x ", c)
    }
  end
  return r
end

# IRVפƤ뤫
def test_irv(fn)
  s = "x"
  (0..127).each {|c|
    s[0] = c
    u = Uconv.__send__(fn[0], s).unpack("S").first
    if u != c
      print "irv: c = #{pb(c)} -> u = #{pb(u)}\n"
      next
    end
    t = Uconv.__send__(fn[1], [u].pack("S"))
    if t != s
      print "irv: u = #{pb(u)} -> t = #{pb(t)}\n"
      next
    end
  }
end

def test_round_trip(fn, table)
  table.each {|s|
    u = Uconv.__send__(fn[0], s).unpack("S").first
    if !u
      print "round: s = #{pb(s)} -> u = (nil)\n"
      next
    end
    t = Uconv.__send__(fn[1], [u].pack("S"))
    if t != s
      print "round: s = #{pb(s)} -> u = #{pb(u)} -> t = #{pb(t)}\n"
      next
    end
  }
end

def test_strict_conv(fn, table)
  table.each {|pair|
    u = Uconv.__send__(fn[0], pair[0]).unpack("S").first
    if !u
      print "strict: s = #{pb(pair[0])} failed\n"
      next
    end
    if u != pair[1]
      printf("strict: s = #{pb(pair[0])} -> expect U+%04x, but U+%04x\n", pair[1], u)
      next
    else
      print "strict: s = #{pb(pair[0])} ok\n"
    end
  }
end

EUC_FN = [:euctou2, :u2toeuc]
SJIS_FN = [:sjistou2, :u2tosjis]

def test_eucjp()
  print "eucjp test.\n"
  test_irv(EUC_FN)
  test_round_trip(EUC_FN, EUCJP_RT_CHARS)
  test_strict_conv(EUC_FN, EUCJP_TOUCS)
end

def test_sjis()
  print "sjis test.\n"
  test_irv(SJIS_FN)
  test_round_trip(SJIS_FN, SJIS_RT_CHARS)
  test_strict_conv(SJIS_FN, SJIS_TOUCS)
end

def main()
  test_eucjp()
  test_sjis()
end
main()
